@@ -1,3 +1,4 @@
+from dataclasses import dataclass
 from websocket import WebSocketApp
 from threading import Thread
 from datetime import datetime
@@ -6,11 +7,15 @@ import time
 import json
 from copy import deepcopy
 from collections import defaultdict
+from .support import full_feed , full_feed_bidask , bar_feed
 
 from colorama import Style, Fore
 
 import traceback
 
+
+
+
 class LiveClient(WebSocketApp):
 
     def __init__(self, parent_app, url, *args):
@@ -32,6 +37,9 @@ class LiveClient(WebSocketApp):
         self.disconnect_flag = False
         self.heartbeat_check_thread = Thread(target=self.check_heartbeat, daemon=True)
         self.trade_callback = None
+        self.full_feed_trade_callback = None
+        self.full_feed_bar_callback = None
+        self.full_feed_bidask_callback = None
         self.one_min_bar_callback = None
         self.five_min_bar_callback = None
         self.bidask_callback = None
@@ -80,9 +88,9 @@ class LiveClient(WebSocketApp):
 
     def reconnect(self):
         self.close()
+        time.sleep(2)
         t = Thread(target=self.run_forever, daemon=True)
         t.start()
-        time.sleep(1)
         is_td_connected = False
         while not is_td_connected:
             time.sleep(self.heartbeat_interval + self.heartbeat_buffer)
@@ -94,16 +102,29 @@ class LiveClient(WebSocketApp):
         msg = json.loads(message)
         msg_keys = msg.keys()
         # print(msg)
+        print(msg)
         if 'message' in msg_keys:
             self.handle_message_data(msg)
-        if 'trade' in msg_keys:
-            self.handle_trade_data( msg['trade'] )
-        elif 'bidask' in msg_keys:
-            self.handle_bid_ask_data( msg['bidask'] )
-        elif 'bar1min' in msg_keys:
-            self.handle_bar_data( msg['bar1min'] , bar_type= '1min' )
-        elif 'bar5min' in msg_keys:
-            self.handle_bar_data( msg['bar5min'] , bar_type= '5min')
+        if 'trade' in msg_keys and self.parent_app.full_feed :
+            feed_tick = full_feed( msg['trade'] , self.parent_app.symbol_id_map_dict[msg['trade'][0]] )
+            self.full_feed_trade_callback( feed_tick ) if self.full_feed_trade_callback else 0   
+        if 'bidask' in msg_keys and self.parent_app.full_feed :
+            bid_ask = full_feed_bidask( msg['bidask'] , self.parent_app.symbol_id_map_dict[msg['bidask'][0]])
+            self.full_feed_bidask_callback( bid_ask ) if self.full_feed_bidask_callback else 0  
+        if 'interval' in msg_keys and self.parent_app.full_feed :
+            bar_data = map(lambda x : bar_feed(x , self.parent_app.symbol_id_map_dict[ str( x[1]) ]), msg["data"]) 
+            for bar in list(bar_data):
+                self.full_feed_bar_callback ( bar ) if self.full_feed_bar_callback else 0
+
+        if not self.parent_app.full_feed:
+            if 'trade' in msg_keys:
+                self.handle_trade_data( msg['trade'] )
+            elif 'bidask' in msg_keys:
+                self.handle_bid_ask_data( msg['bidask'] )
+            elif 'bar1min' in msg_keys:
+                self.handle_bar_data( msg['bar1min'] , bar_type= '1min' )
+            elif 'bar5min' in msg_keys:
+                self.handle_bar_data( msg['bar5min'] , bar_type= '5min')
 
     def handle_message_data(self, msg):
         if msg['success']:
@@ -311,6 +332,7 @@ class LiveClient(WebSocketApp):
         self.sock.ping()
         self.sock.settimeout(15)
         if self.parent_app.symbol_mkt_id_map:
+            self.logger.info("connection dropped due to timeout. trying reconnecting.....")
             old_symbol_map = temp_symbol_map = deepcopy(self.parent_app.symbol_mkt_id_map)  # temp_symbol_map is destructed while restarting the data
             temp_symbol_map = dict(temp_symbol_map)
             self.parent_app.symbol_mkt_id_map = defaultdict(set)
@@ -318,6 +340,8 @@ class LiveClient(WebSocketApp):
             while temp_symbol_map:
                 restart_symbols = list(temp_symbol_map.keys())
                 restart_req_ids = [temp_symbol_map[symbol].pop() for symbol in restart_symbols]
+                self.sock.ping()
+                time.sleep(3)
                 self.parent_app.start_live_data(restart_symbols, req_id=restart_req_ids)
                 temp_symbol_map = {symbol: symbol_set for (symbol, symbol_set) in temp_symbol_map.items() if symbol_set}
             assert old_symbol_map == self.parent_app.symbol_mkt_id_map
@@ -343,3 +367,5 @@ class LiveClient(WebSocketApp):
             except Exception as e:
                 self.logger.error(f'{type(e)} in reconnection2 => {e}')
         # self.logger.error(f'CLOSE: {self.last_ping_tm} -> {self.last_pong_tm}')
+
+
